@@ -36,8 +36,9 @@ from collections import Mapping
 from datetime import timedelta
 
 from pyudev._libudev import libudev
-from pyudev._util import (assert_unicode, assert_bytes, udev_list_iterate,
-                          property_value_to_bytes, string_to_bool)
+from pyudev._util import (ensure_unicode_string, ensure_byte_string,
+                          udev_list_iterate, property_value_to_bytes,
+                          string_to_bool)
 
 
 __all__ = [
@@ -72,7 +73,7 @@ def udev_version():
     """
     command = ['udevadm', '--version']
     udevadm = Popen(command, stdout=PIPE)
-    stdout = assert_unicode(udevadm.communicate()[0]).strip()
+    stdout = ensure_unicode_string(udevadm.communicate()[0]).strip()
     if udevadm.returncode != 0:
         raise CalledProcessError(0, command)
     return int(stdout)
@@ -105,7 +106,7 @@ class Context(object):
         The mount point can be overwritten using the environment variable
         :envvar:`SYSFS_PATH`.  Use this for testing purposes.
         """
-        return assert_unicode(libudev.udev_get_sys_path(self._context))
+        return ensure_unicode_string(libudev.udev_get_sys_path(self._context))
 
     @property
     def device_path(self):
@@ -114,7 +115,28 @@ class Context(object):
 
         This can be overridden in the udev configuration.
         """
-        return assert_unicode(libudev.udev_get_dev_path(self._context))
+        return ensure_unicode_string(libudev.udev_get_dev_path(self._context))
+
+    @property
+    def log_priority(self):
+        """
+        The logging priority of the interal logging facitility of udev as
+        integer with a standard :mod:`syslog` priority.  Assign to this
+        property to change the logging priority.
+
+        UDev uses the standard :mod:`syslog` priorities.  Constants for these
+        priorities are defined in the :mod:`syslog` module in the standard
+        library:
+
+        >>> import syslog
+        >>> context = pyudev.Context()
+        >>> context.log_priority = syslog.LOG_DEBUG
+        """
+        return libudev.udev_get_log_priority(self._context)
+
+    @log_priority.setter
+    def log_priority(self, value):
+        libudev.udev_set_log_priority(self._context, value)
 
     def list_devices(self, **kwargs):
         """
@@ -222,7 +244,7 @@ class Enumerator(object):
         Return the instance again.
         """
         libudev.udev_enumerate_add_match_subsystem(
-            self._enumerator, assert_bytes(subsystem))
+            self._enumerator, ensure_byte_string(subsystem))
         return self
 
     def match_sys_name(self, sys_name):
@@ -236,7 +258,7 @@ class Enumerator(object):
         .. versionadded:: 0.8
         """
         libudev.udev_enumerate_add_match_sysname(self._enumerator,
-                                                 assert_bytes(sys_name))
+                                                 ensure_byte_string(sys_name))
         return self
 
     def match_property(self, property, value):
@@ -255,7 +277,7 @@ class Enumerator(object):
         Return the instance again.
         """
         libudev.udev_enumerate_add_match_property(
-            self._enumerator, assert_bytes(property),
+            self._enumerator, ensure_byte_string(property),
             property_value_to_bytes(value))
         return self
 
@@ -272,7 +294,7 @@ class Enumerator(object):
         .. udevminversion:: 154
         """
         libudev.udev_enumerate_add_match_tag(self._enumerator,
-                                             assert_bytes(tag))
+                                             ensure_byte_string(tag))
         return self
 
     def match_is_initialized(self):
@@ -454,7 +476,8 @@ class Device(Mapping):
         device inside ``sysfs`` with the mount point included.
 
         Return a :class:`Device` object for the device.  Raise
-        :exc:`DeviceNotFoundAtPathError`, if no device was found for ``sys_path``.
+        :exc:`DeviceNotFoundAtPathError`, if no device was found for
+        ``sys_path``.
 
         .. versionchanged:: 0.4
            Raise :exc:`NoSuchDeviceError` instead of returning ``None``, if
@@ -464,7 +487,7 @@ class Device(Mapping):
            :exc:`NoSuchDeviceError`
         """
         device = libudev.udev_device_new_from_syspath(
-            context._context, assert_bytes(sys_path))
+            context._context, ensure_byte_string(sys_path))
         if not device:
             raise DeviceNotFoundAtPathError(sys_path)
         return cls(context, device)
@@ -492,8 +515,8 @@ class Device(Mapping):
         .. versionadded:: 0.5
         """
         device = libudev.udev_device_new_from_subsystem_sysname(
-            context._context, assert_bytes(subsystem),
-            assert_bytes(sys_name))
+            context._context, ensure_byte_string(subsystem),
+            ensure_byte_string(sys_name))
         if not device:
             raise DeviceNotFoundByNameError(subsystem, sys_name)
         return cls(context, device)
@@ -565,6 +588,34 @@ class Device(Mapping):
         for device in self.context.list_devices().match_children(self):
             yield device
 
+    def find_parent(self, subsystem, device_type=None):
+        """
+        Find the parent device with the given ``subsystem`` and
+        ``device_type``.
+
+        ``subsystem`` is a byte or unicode string containing the name of the
+        subsystem, in which to search for the parent.  ``device_type`` is a
+        byte or unicode string holding the expected device type of the parent.
+        It can be ``None`` (the default), which means, that no specific device
+        type is expected.
+
+        Return a parent :class:`Device` within the given ``subsystem`` and – if
+        ``device_type`` is not ``None`` – with the given ``device_type``, or
+        ``None``, if this device has no parent device matching these
+        constraints.
+
+        .. versionadded:: 0.9
+        """
+        subsystem = ensure_byte_string(subsystem)
+        if device_type is not None:
+            device_type = ensure_byte_string(device_type)
+        parent = libudev.udev_device_get_parent_with_subsystem_devtype(
+            self._device, subsystem, device_type)
+        if not parent:
+            return None
+        # parent device is not referenced, thus forcibly acquire a reference
+        return Device(self.context, libudev.udev_device_ref(parent))
+
     def traverse(self):
         """
         Traverse all parent devices of this device from bottom to top.
@@ -584,7 +635,8 @@ class Device(Mapping):
         Absolute path of this device in ``sysfs`` including the ``sysfs``
         mount point as unicode string.
         """
-        return assert_unicode(libudev.udev_device_get_syspath(self._device))
+        return ensure_unicode_string(
+            libudev.udev_device_get_syspath(self._device))
 
     @property
     def device_path(self):
@@ -596,21 +648,24 @@ class Device(Mapping):
         mount point.  However, the path is absolute and starts with a slash
         ``'/'``.
         """
-        return assert_unicode(libudev.udev_device_get_devpath(self._device))
+        return ensure_unicode_string(
+            libudev.udev_device_get_devpath(self._device))
 
     @property
     def subsystem(self):
         """
         Name of the subsystem this device is part of as unicode string.
         """
-        return assert_unicode(libudev.udev_device_get_subsystem(self._device))
+        return ensure_unicode_string(
+            libudev.udev_device_get_subsystem(self._device))
 
     @property
     def sys_name(self):
         """
         Device file name inside ``sysfs`` as unicode string.
         """
-        return assert_unicode(libudev.udev_device_get_sysname(self._device))
+        return ensure_unicode_string(
+            libudev.udev_device_get_sysname(self._device))
 
     @property
     def driver(self):
@@ -622,7 +677,7 @@ class Device(Mapping):
         """
         driver = libudev.udev_device_get_driver(self._device)
         if driver:
-            return assert_unicode(driver)
+            return ensure_unicode_string(driver)
 
     @property
     def device_node(self):
@@ -638,7 +693,7 @@ class Device(Mapping):
         """
         node = libudev.udev_device_get_devnode(self._device)
         if node:
-            return assert_unicode(node)
+            return ensure_unicode_string(node)
 
     @property
     def is_initialized(self):
@@ -700,7 +755,7 @@ class Device(Mapping):
         """
         entry = libudev.udev_device_get_devlinks_list_entry(self._device)
         for name in udev_list_iterate(entry):
-            yield assert_unicode(name)
+            yield ensure_unicode_string(name)
 
     @property
     def attributes(self):
@@ -732,7 +787,7 @@ class Device(Mapping):
         """
         entry = libudev.udev_device_get_tags_list_entry(self._device)
         for tag in udev_list_iterate(entry):
-            yield assert_unicode(tag)
+            yield ensure_unicode_string(tag)
 
     def __iter__(self):
         """
@@ -743,7 +798,7 @@ class Device(Mapping):
         """
         entry = libudev.udev_device_get_properties_list_entry(self._device)
         for name in udev_list_iterate(entry):
-            yield assert_unicode(name)
+            yield ensure_unicode_string(name)
 
     def __len__(self):
         """
@@ -767,10 +822,10 @@ class Device(Mapping):
         for this device.
         """
         value = libudev.udev_device_get_property_value(
-            self._device, assert_bytes(property))
+            self._device, ensure_byte_string(property))
         if value is None:
             raise KeyError('No such property: {0}'.format(property))
-        return assert_unicode(value)
+        return ensure_unicode_string(value)
 
     def asint(self, property):
         """
@@ -888,7 +943,7 @@ class Attributes(Mapping):
 
     def __contains__(self, attribute):
         return libudev.udev_device_get_sysattr_value(
-            self.device._device, assert_bytes(attribute)) is not None
+            self.device._device, ensure_byte_string(attribute)) is not None
 
     def __getitem__(self, attribute):
         """
@@ -902,7 +957,7 @@ class Attributes(Mapping):
         for this device.
         """
         value = libudev.udev_device_get_sysattr_value(
-            self.device._device, assert_bytes(attribute))
+            self.device._device, ensure_byte_string(attribute))
         if value is None:
             raise KeyError('No such attribute: {0}'.format(attribute))
         return value
@@ -922,7 +977,7 @@ class Attributes(Mapping):
         for this device, or :exc:`~exceptions.UnicodeDecodeError`, if the
         content of the attribute cannot be decoded into a unicode string.
         """
-        return assert_unicode(self[attribute])
+        return ensure_unicode_string(self[attribute])
 
     def asint(self, attribute):
         """
