@@ -34,7 +34,7 @@ import select
 from contextlib import closing
 
 from pyudev._libudev import libudev, get_libudev_errno
-from pyudev._util import assert_bytes, assert_unicode
+from pyudev._util import ensure_byte_string, ensure_unicode_string
 
 from pyudev.core import Device
 
@@ -108,7 +108,7 @@ class Monitor(object):
         if source not in ('kernel', 'udev'):
             raise ValueError('Invalid source: {0!r}. Must be one of "udev" '
                              'or "kernel"'.format(source))
-        source = assert_bytes(source)
+        source = ensure_byte_string(source)
         monitor = libudev.udev_monitor_new_from_netlink(
             context._context, source)
         if not monitor:
@@ -120,21 +120,20 @@ class Monitor(object):
         """
         Connect to an arbitrary udev daemon using the given ``socket_path``.
 
-        ``context`` is the :class:`Context` to use. ``socket_path`` is a
-        byte or unicode string, pointing to an existing socket.  If the path
-        starts with a @, use an abstract namespace socket.  If
-        ``socket_path`` does not exist, fall back to an abstract namespace
-        socket.
+        ``context`` is the :class:`Context` to use. ``socket_path`` is a byte
+        or unicode string, pointing to an existing socket.  If the path starts
+        with a ``@``, use an abstract namespace socket.  If ``socket_path``
+        does not exist, fall back to an abstract namespace socket.
 
         The caller is responsible for permissions and cleanup of the socket
         file.
 
-        Return a new :class:`Monitor` object, which is connected to the
-        given socket.  Raise :exc:`~exceptions.EnvironmentError`, if the
-        creation of the monitor failed.
+        Return a new :class:`Monitor` object, which is connected to the given
+        socket.  Raise :exc:`~exceptions.EnvironmentError`, if the creation of
+        the monitor failed.
         """
         monitor = libudev.udev_monitor_new_from_socket(
-            context._context, assert_bytes(socket_path))
+            context._context, ensure_byte_string(socket_path))
         if not monitor:
             raise EnvironmentError('Could not create monitor for socket: '
                                    '{0!r}'.format(socket_path))
@@ -168,12 +167,33 @@ class Monitor(object):
 
         This method must be called *before* :meth:`enable_receiving`.
         """
-        subsystem = assert_bytes(subsystem)
+        subsystem = ensure_byte_string(subsystem)
         if device_type:
-            device_type = assert_bytes(device_type)
+            device_type = ensure_byte_string(device_type)
         libudev.udev_monitor_filter_add_match_subsystem_devtype(
             self._monitor, subsystem, device_type)
 
+    def filter_by_tag(self, tag):
+        """
+        Filter incoming events by the given ``tag``.
+
+        ``tag`` is a byte or unicode string with the name of a tag.  Only
+        events for devices which have this tag attached pass the filter and are
+        handed to the caller.
+
+        Like with :meth:`filter_by` this filter is also executed inside the
+        kernel, so that client processes are usually not woken up for devices
+        without the given ``tag``.
+
+        This method must be called *before* :meth:`enable_receiving`.
+
+        .. udevminversion:: 154
+
+        .. versionadded:: 0.9
+        """
+        libudev.udev_monitor_filter_add_match_tag(
+            self._monitor, ensure_byte_string(tag))
+
     def enable_receiving(self):
         """
         Switch the monitor into listing mode.
@@ -226,7 +246,8 @@ class Monitor(object):
                 raise EnvironmentError('Could not receive device')
             else:
                 raise EnvironmentError(errno, os.strerror(errno))
-        action = assert_unicode(libudev.udev_device_get_action(device_p))
+        action = ensure_unicode_string(
+            libudev.udev_device_get_action(device_p))
         return action, Device(self.context, device_p)
 
     def __iter__(self):
