@@ -26,9 +26,12 @@ from itertools import count
 from datetime import timedelta
 
 import pytest
+import mock
 
 from pyudev import (Device,
-                    DeviceNotFoundAtPathError, DeviceNotFoundByNameError)
+                    DeviceNotFoundAtPathError,
+                    DeviceNotFoundByNameError,
+                    DeviceNotFoundInEnvironmentError)
 
 
 def pytest_generate_tests(metafunc):
@@ -100,20 +103,19 @@ def test_device_from_name_nonexisting_subsystem(context):
 
 @pytest.check_udev_version('>= 152')
 def test_device_from_environment(context):
-    raise NotImplementedError()
+    # there is no device in a standard environment
+    with pytest.raises(DeviceNotFoundInEnvironmentError):
+        Device.from_environment(context)
 
 
 @pytest.mark.properties
 def test_device_properties(device, properties):
-    properties.pop('DEVNAME', None)
-    for n, property in enumerate(properties, start=1):
-        assert device[property] == properties[property]
-    assert n > 0
+    assert all(device[p] == properties[p] for p in properties)
 
 
 @pytest.mark.properties
 def test_device_asint(device, properties):
-    for n, property in enumerate(properties, start=1):
+    for property in properties:
         value = properties[property]
         try:
             value = int(value)
@@ -122,12 +124,11 @@ def test_device_asint(device, properties):
                 device.asint(property)
         else:
             assert device.asint(property) == value
-    assert n > 0
 
 
 @pytest.mark.properties
 def test_device_asbool(device, properties):
-    for n, property in enumerate(properties, start=1):
+    for property in properties:
         value = properties[property]
         if value == '1':
             assert device.asbool(property)
@@ -138,13 +139,11 @@ def test_device_asbool(device, properties):
                 device.asbool(property)
             message = 'Not a boolean value: {0!r}'
             assert str(exc_info.value) == message.format(value)
-    assert n > 0
 
 
 def test_attributes_iter(device, attributes):
     device_attributes = set(device.attributes)
-    for attribute in attributes:
-        assert attribute in device_attributes
+    assert all(a in device_attributes for a in attributes)
     assert all(pytest.is_unicode_string(a) for a in device_attributes)
 
 
@@ -160,18 +159,16 @@ def test_attributes_contains(device, attributes):
 
 
 def test_attributes_getitem(device, attributes):
-    for attribute, value in attributes.items():
-        # device attributes *must* be bytes.
-        assert isinstance(device.attributes[attribute], bytes)
-        value = value.encode(sys.getfilesystemencoding())
-        assert device.attributes[attribute] == value
+    assert all(isinstance(device.attributes[a], bytes) for a in attributes)
+    assert all(device.attributes[a] == v.encode(sys.getfilesystemencoding())
+               for a, v in attributes.items())
 
 
 def test_attributes_asstring(device, attributes):
-    for attribute, value in attributes.items():
-        assert pytest.is_unicode_string(
-            device.attributes.asstring(attribute))
-        assert device.attributes.asstring(attribute) == value
+    assert all(pytest.is_unicode_string(device.attributes.asstring(a))
+               for a in attributes)
+    assert all(device.attributes.asstring(a) == v
+               for a, v in attributes.items())
 
 
 def test_attributes_asint(device, attributes):
@@ -200,11 +197,10 @@ def test_attributes_asbool(device, attributes):
 
 @pytest.mark.properties
 def test_device_devname(context, device, all_properties):
-    if 'DEVNAME' not in device:
+    if 'DEVNAME' not in all_properties:
         pytest.xfail('%r has no DEVNAME' % device)
-    assert device['DEVNAME'].startswith(context.device_path)
-    assert device['DEVNAME'] == os.path.join(context.device_path,
-                                             all_properties['DEVNAME'])
+    assert os.path.join(context.device_path, device['DEVNAME']) == \
+           os.path.join(context.device_path, all_properties['DEVNAME'])
 
 
 @pytest.mark.properties
@@ -240,23 +236,56 @@ def test_device_links(context, device, device_links):
 @pytest.mark.properties
 def test_device_is_initialized(device):
     assert isinstance(device.is_initialized, bool)
-    with pytest.patch_libudev('udev_device_get_is_initialized') as func:
-        func.return_value = True
+    get_is_initialized = 'udev_device_get_is_initialized'
+    with pytest.patch_libudev(get_is_initialized) as get_is_initialized:
+        get_is_initialized.return_value = True
         assert device.is_initialized
-        assert func.called
+        get_is_initialized.assert_called_with(device._device)
 
 
 @pytest.check_udev_version('>= 165')
 @pytest.mark.properties
 def test_device_time_since_initialized(device):
     assert isinstance(device.time_since_initialized, timedelta)
-    funcname = 'udev_device_get_usec_since_initialized'
-    with pytest.patch_libudev(funcname) as func:
-        func.return_value = 100
+    usec_since_init = 'udev_device_get_usec_since_initialized'
+    with pytest.patch_libudev(usec_since_init) as usec_since_init:
+        usec_since_init.return_value = 100
         assert device.time_since_initialized.microseconds == 100
-        assert func.called
+        usec_since_init.assert_called_with(device._device)
 
 
+@pytest.check_udev_version('>= 154')
+def test_device_tags_mock(device):
+    tags_list = iter([b'spam', b'eggs', b'foo', b'bar'])
+
+    def next_entry(entry):
+        try:
+            return next(tags_list)
+        except StopIteration:
+            return None
+    def name(entry):
+        if entry:
+            return entry
+        else:
+            pytest.fail('empty entry!')
+
+    get_tags_list_entry = 'udev_device_get_tags_list_entry'
+    get_next = 'udev_list_entry_get_next'
+    get_name = 'udev_list_entry_get_name'
+    with pytest.nested(pytest.patch_libudev(get_tags_list_entry),
+                       pytest.patch_libudev(get_name),
+                       pytest.patch_libudev(get_next)) as (get_tags_list_entry,
+                                                           get_name, get_next):
+        get_tags_list_entry.return_value = next(tags_list)
+        get_name.side_effect = name
+        get_next.side_effect = next_entry
+
+        device_tags = list(device.tags)
+        assert device_tags == ['spam', 'eggs', 'foo', 'bar']
+        assert all(pytest.is_unicode_string(t) for t in device_tags)
+
+
+@pytest.mark.xfail(reason='Not implemented')
 @pytest.check_udev_version('>= 154')
 def test_device_tags():
     raise NotImplementedError()
@@ -284,6 +313,46 @@ def test_device_traverse(device):
         child = parent
 
 
+def test_device_find_parent(device):
+    parent = device.find_parent(device.subsystem)
+    if not parent:
+        pytest.xfail('no parent within the same subsystem')
+    assert parent.subsystem == device.subsystem
+    assert parent in device.traverse()
+    assert device in parent.children
+
+
+def test_device_find_parent_no_devtype_mock(device):
+    get_parent = 'udev_device_get_parent_with_subsystem_devtype'
+    ref = 'udev_device_ref'
+    with pytest.nested(pytest.patch_libudev(get_parent),
+                       pytest.patch_libudev(ref)) as (get_parent, ref):
+        get_parent.return_value = mock.sentinel.device
+        ref.return_value = mock.sentinel.referenced_device
+        parent = device.find_parent('subsystem')
+        get_parent.assert_called_with(device._device, b'subsystem', None)
+        ref.assert_called_with(mock.sentinel.device)
+        assert isinstance(get_parent.call_args[0][1], bytes)
+        assert isinstance(parent, Device)
+        assert parent._device is mock.sentinel.referenced_device
+
+
+def test_device_find_parent_with_devtype_mock(device):
+    get_parent = 'udev_device_get_parent_with_subsystem_devtype'
+    ref = 'udev_device_ref'
+    with pytest.nested(pytest.patch_libudev(get_parent),
+                       pytest.patch_libudev(ref)) as (get_parent, ref):
+        get_parent.return_value = mock.sentinel.device
+        ref.return_value = mock.sentinel.referenced_device
+        parent = device.find_parent('subsystem', 'devtype')
+        get_parent.assert_called_with(device._device, b'subsystem', b'devtype')
+        ref.assert_called_with(mock.sentinel.device)
+        args = get_parent.call_args[0][1:]
+        assert all(isinstance(a, bytes) for a in args)
+        assert isinstance(parent, Device)
+        assert parent._device is mock.sentinel.referenced_device
+
+
 @pytest.mark.operator
 def test_device_eq(device):
     assert device == device.device_path
